"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.onPostBuild = exports.sourceNodes = exports.onCreatePage = exports.createPages = exports.createSchemaCustomization = exports.onCreateWebpackConfig = exports.onPreBootstrap = void 0;
const fs_1 = __importDefault(require("fs"));
const jaen_utils_1 = require("jaen-utils");
const path_1 = __importDefault(require("path"));
const index_js_1 = require("./search/index.js");
const iam_process_js_1 = require("./iam-process.js");
const onPreBootstrap = async ({ reporter }) => {
    await checkSourceFolder(jaen_utils_1.JaenSource.sourcePagesPath);
    await checkSourceFolder(jaen_utils_1.JaenSource.sourceTemplatesPath);
    async function checkSourceFolder(sourcePath) {
        if (sourcePath) {
            reporter.info(`Found jaen-pages at ${sourcePath}`);
            const dir = await fs_1.default.promises.opendir(sourcePath);
            for await (const dirent of dir) {
                if (dirent.isFile() &&
                    ['.tsx', '.ts', '.js', '.jsx'].includes(path_1.default.extname(dirent.name))) {
                    const pagePath = path_1.default.join(sourcePath, dirent.name);
                    const moduleText = await fs_1.default.promises.readFile(pagePath, 'utf8');
                    if (!moduleText.includes('export const query = graphql`')) {
                        reporter.warn(`The page ${pagePath} does not contain a graphql export.\n` +
                            `Note: The graphql export must be in the form of 'export const query = graphql' and must contain the correct graphql query for jaen pages.`);
                    }
                }
            }
        }
    }
};
exports.onPreBootstrap = onPreBootstrap;
const onCreateWebpackConfig = ({ actions, loaders, plugins, stage }, pluginOptions) => {
    const snekResourceId = pluginOptions.snekResourceId;
    if (!snekResourceId) {
        throw new Error(`The plugin option 'snekResourceId' is required. Please add the option to your gatsby-config.js file.`);
    }
    actions.setWebpackConfig({
        plugins: [
            plugins.define({
                ___JAEN_SOURCE_TEMPLATES___: JSON.stringify(jaen_utils_1.JaenSource.sourceTemplatesPath),
                ___JAEN_SOURCE_PAGES___: JSON.stringify(jaen_utils_1.JaenSource.sourcePagesPath),
                ___JAEN_SOURCE_VIEWS___: JSON.stringify(jaen_utils_1.JaenSource.sourceViewsPath),
                ___JAEN_SOURCE_POPUPS___: JSON.stringify(jaen_utils_1.JaenSource.sourcePopupsPath),
                ___SNEK_RESOURCE_ID___: JSON.stringify(snekResourceId)
            })
        ]
    });
    if (stage === 'build-html' || stage === 'develop-html') {
        actions.setWebpackConfig({
            module: {
                rules: [
                    {
                        test: /canvas/,
                        use: loaders.null()
                    },
                    {
                        test: /filerobot-image-editor/,
                        use: loaders.null()
                    }
                ]
            }
        });
    }
};
exports.onCreateWebpackConfig = onCreateWebpackConfig;
const createSchemaCustomization = ({ actions }) => {
    actions.createFieldExtension({
        name: 'file',
        args: {},
        extend(_options, _prevFieldConfig) {
            return {
                args: {},
                resolve(source, _args, context, info) {
                    const fieldName = info.fieldName;
                    const fieldPathKey = info.path.key;
                    if (fieldPathKey === info.fieldName) {
                        throw new Error(`The fieldKey ${fieldPathKey} is the same as the fieldName ${fieldName}, please set the fieldKey to the correct fieldName of an ImageField.`);
                    }
                    const imageId = source.jaenFields?.['IMA:ImageField']?.[fieldPathKey]?.value
                        ?.imageId;
                    const node = context.nodeModel.getNodeById({
                        id: imageId,
                        type: 'File'
                    });
                    return node;
                }
            };
        }
    });
    actions.createFieldExtension({
        name: 'buildPath',
        args: {},
        extend(_options, _prevFieldConfig) {
            return {
                args: {},
                async resolve(source, _args, context, _info) {
                    const items = Array.from((await context.nodeModel.findAll({ type: `JaenPage` })).entries);
                    const originPath = (0, jaen_utils_1.generatePageOriginPath)(items.map((item) => ({
                        id: item.id,
                        slug: item.slug,
                        parent: item.parent ? { id: item.parent } : null
                    })), {
                        id: source.id,
                        slug: source.slug,
                        parent: source.parent
                            ? {
                                id: source.parent
                            }
                            : null
                    });
                    return originPath;
                }
            };
        }
    });
    actions.createFieldExtension({
        name: 'componentName',
        args: {},
        extend(_options, _prevFieldConfig) {
            return {
                args: {},
                async resolve(source, _args, context, _info) {
                    const items = Array.from((await context.nodeModel.findAll({ type: `SitePage` })).entries);
                    const page = items.find((item) => item.context.jaenPageId === source.id);
                    const sourcePages = jaen_utils_1.JaenSource.sourcePagesPath;
                    const componentName = page?.component?.includes(sourcePages)
                        ? page.component.replace(`${sourcePages}/`, '')
                        : undefined;
                    return componentName;
                }
            };
        }
    });
    actions.createTypes(`
    type JaenWidget {
      name: String!
      data: JSON
    }

    type RemoteFileMigration {
      createdAt: Date!
      fileUrl: String!
    }

    type JaenInternal implements Node {
      finderUrl: String
      widgets: [JaenWidget!]!
      migrationHistory: [RemoteFileMigration!]!
      siteMetadata: JaenSiteMetadata
    }

    type JaenPopup implements Node {
      id: ID!
      jaenFields: JSON
      active: Boolean
    }

    type JaenPage implements Node {
      id: ID!
      slug: String!
      jaenPageMetadata: JaenPageMetadata!
      jaenFields: JSON
      jaenFile: File @file
      jaenFiles: [File] @link

      sections: [JaenSection!]!

      template: String
      buildPath: String @buildPath
      componentName: String @componentName
      excludedFromIndex: Boolean
    }

    type JaenSection {
      fieldName: String!
      items: [JaenSectionItem!]!
      ptrHead: String
      ptrTail: String
    }

    type JaenSectionItem {
      id: ID!
      type: String!
      ptrPrev: String
      ptrNext: String
      jaenFields: JSON
      jaenFiles: [File] @link
      jaenFile: File @file

      sections: [JaenSection!]!
    }

    type JaenSectionPath {
      fieldName: String!
      sectionId: String
    }


    type JaenPageMetadata {
      title: String!
      description: String
      image: String
      canonical: String
      datePublished: String
      isBlogPost: Boolean
    }

    type JaenSiteMetadata {
      siteUrl: String
      title: String
      description: String
      image: String
      author: JaenSiteMetadataAuthor
      organization: JaenSiteMetadataOrganization
      social: JaenSiteMetadataSocial
    }
    
    type JaenSiteMetadataAuthor {
      name: String
    }

    type JaenSiteMetadataOrganization {
      name: String
      url: String
      logo: String
    }

    type JaenSiteMetadataSocial {
      twitter: String
      fbAppID: String
    }
  `);
};
exports.createSchemaCustomization = createSchemaCustomization;
const createPages = async ({ actions, graphql, reporter, getNode }) => {
    const { createPage, createNodeField } = actions;
    createPage({
        path: '/admin',
        matchPath: '/admin/*',
        component: require.resolve('../AdminPage.tsx'),
        context: {}
    });
    createPage({
        path: '/admin/login',
        component: require.resolve('../LoginPage.tsx'),
        context: {}
    });
    const createJaenPages = async () => {
        const result = await graphql(`
      query {
        allTemplate: allFile(
          filter: {sourceInstanceName: {eq: "jaen-templates"}}
        ) {
          nodes {
            name
            absolutePath
          }
        }
        allJaenPage {
          nodes {
            id
            slug
            parent {
              id
            }
            template
            jaenPageMetadata {
              title
            }
            jaenFields
          }
        }
      }
    `);
        if (result.errors || !result.data) {
            reporter.panicOnBuild(`Error while running GraphQL query. ${result.errors}`);
            return;
        }
        const { allTemplate, allJaenPage } = result.data;
        for (const node of allJaenPage.nodes) {
            const pagePath = (0, jaen_utils_1.generatePageOriginPath)(allJaenPage.nodes, node);
            const pureNode = getNode(node.id);
            if (pureNode) {
                createNodeField({
                    node: pureNode,
                    name: 'path',
                    value: pagePath
                });
            }
            if (node.template) {
                if (!pagePath) {
                    reporter.panicOnBuild(`Error while generating path for page ${node.id}`);
                    return;
                }
                const template = allTemplate.nodes.find(template => template.name === node.template);
                if (!template) {
                    reporter.panicOnBuild(`Template ${node.template} not found`);
                    return;
                }
                createPage({
                    path: pagePath,
                    component: template.absolutePath,
                    context: {
                        jaenPageId: node.id
                    }
                });
            }
        }
    };
    await createJaenPages();
};
exports.createPages = createPages;
const onCreatePage = async ({ actions, page, createContentDigest, getNode }) => {
    const { createPage, deletePage, createNode } = actions;
    let stepPage = page;
    const blacklist = ['/admin'];
    if (blacklist.includes(page.path)) {
        return;
    }
    if (!page.context?.jaenPageId) {
        if (!page.context?.skipJaenPage) {
            const jaenPageId = `JaenPage ${page.path}`;
            const slugifiedPath = (0, jaen_utils_1.convertToSlug)(page.path);
            const existingNode = getNode(jaenPageId);
            if (!existingNode) {
                const jaenPage = {
                    id: jaenPageId,
                    slug: slugifiedPath,
                    parent: null,
                    children: [],
                    jaenPageMetadata: {
                        title: page.path
                    },
                    jaenFields: null,
                    jaenFiles: [],
                    sections: [],
                    template: null
                };
                await createNode({
                    ...jaenPage,
                    parent: null,
                    children: [],
                    jaenFiles: [],
                    internal: {
                        type: 'JaenPage',
                        content: JSON.stringify(jaenPage),
                        contentDigest: createContentDigest(jaenPage)
                    }
                });
            }
            stepPage = { ...stepPage, context: { ...page.context, jaenPageId } };
            deletePage(page);
            createPage(stepPage);
        }
    }
};
exports.onCreatePage = onCreatePage;
const sourceNodes = async ({ actions, createNodeId, createContentDigest, cache, store, reporter }) => {
    const { createNode } = actions;
    jaen_utils_1.JaenSource.jaenData.read();
    const internalData = jaen_utils_1.JaenSource.jaenData.internal || {};
    const internalNode = {
        ...internalData,
        id: createNodeId(`internal`),
        parent: null,
        children: [],
        internal: {
            type: `JaenInternal`,
            content: JSON.stringify(internalData),
            contentDigest: createContentDigest(internalData)
        }
    };
    await createNode(internalNode);
    for (const [id, jaenPage] of Object.entries(jaen_utils_1.JaenSource.jaenData.pages)) {
        const page = await jaenPage.context.fetchRemoteFile();
        if (page.deleted) {
            continue;
        }
        await (0, iam_process_js_1.processPage)({
            page,
            createNodeId,
            createNode,
            cache,
            store,
            reporter
        });
        const path = id.split('JaenPage ')[1];
        const node = {
            ...page,
            jaenPageMetadata: {
                ...page.jaenPageMetadata,
                title: page.jaenPageMetadata?.title || path
            },
            slug: page.slug || (path ? (0, jaen_utils_1.convertToSlug)(path) : ''),
            id,
            template: page.template || null,
            componentName: page.componentName || null,
            sections: page.sections || [],
            parent: page.parent ? page.parent.id : null,
            children: page.children?.map(child => child.id) || [],
            internal: {
                type: 'JaenPage',
                content: JSON.stringify(page),
                contentDigest: createContentDigest(page)
            }
        };
        await createNode(node);
    }
};
exports.sourceNodes = sourceNodes;
const onPostBuild = async ({ graphql, reporter }) => {
    const result = await graphql(`
    query {
      allJaenPage {
        nodes {
          id
          slug
          parent {
            id
          }
          template
          componentName
          jaenPageMetadata {
            title
          }
          jaenFields
        }
      }
    }
  `);
    if (result.errors || !result.data) {
        reporter.panicOnBuild(`Error while running GraphQL query. ${result.errors}`);
        return;
    }
    const { allJaenPage } = result.data;
    await preparePagesAndBuildSearch(allJaenPage);
};
exports.onPostBuild = onPostBuild;
async function preparePagesAndBuildSearch(allJaenPage) {
    const nodesForSearchIndex = allJaenPage.nodes.map(node => {
        const originPath = (0, jaen_utils_1.generatePageOriginPath)(allJaenPage.nodes, node) || '';
        let type = node.template || node.componentName || undefined;
        if (type && path_1.default.extname(type)) {
            type = path_1.default.basename(type, path_1.default.extname(type));
        }
        return {
            id: node.id,
            path: originPath,
            jaenPageMetadata: node.jaenPageMetadata,
            jaenFields: node.jaenFields,
            type
        };
    });
    console.log('nodesForSearchIndex', nodesForSearchIndex, allJaenPage.nodes);
    const searchIndex = await (0, index_js_1.buildSearchIndex)(nodesForSearchIndex);
    console.log('searchIndex', searchIndex);
    await fs_1.default.promises.writeFile(path_1.default.join('public', 'search-index.json'), JSON.stringify(searchIndex));
}
//# sourceMappingURL=gatsby-node.js.map